/*
    RECENT ITEMS (Version 4), an extension for the gnome-shell.
    (C) 2011, 2012 Kurt Fleisch; <http://www.bananenfisch.net/gnome/>
    Gnome Shell Extensions: <https://extensions.gnome.org/>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version. <http://www.gnu.org/licenses/>


    DEFINE YOUR SETTINGS HERE (restart the gnome-shell after changing...)
*/
const ITEMS = 10;       // number of items to list
const MORE = 50;        // number of items to list under "more..."
const BLACKLIST = "";   // to blacklist (hide) spezific MIME media types

/*
    available media-types are: text, image, audio, video, application, multipart, message, model.
    you can define one or more (seperate with ",") types.

    for example to blacklist all images use:
const BLACKLIST = "image";

    to blacklist all images, videos and audios, you can use:
const BLACKLIST = "image,audio,video";

    END OF SETTINGS SECTION
*/

const DocInfo = imports.misc.docInfo;
const Gtk = imports.gi.Gtk;
const Gio = imports.gi.Gio;
const St = imports.gi.St;
const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;
const PanelMenu = imports.ui.panelMenu;
const Lang = imports.lang;

function MyPopupMenuItem()
{
  this._init.apply(this, arguments);
}

MyPopupMenuItem.prototype =
{
    __proto__: PopupMenu.PopupBaseMenuItem.prototype,

    _init: function(icon, text, params)
    {
        PopupMenu.PopupBaseMenuItem.prototype._init.call(this, params);

        this.box = new St.BoxLayout({ style_class: 'popup-combobox-item' });

        this.icon = icon;
        this.box.add(this.icon);
        this.label = new St.Label({ text: text });
        this.box.add(this.label);
        this.addActor(this.box);
    }
};

function RecentItems()
{
  this._init.apply(this, arguments);
}

RecentItems.prototype =
{
    __proto__: PanelMenu.Button.prototype,
 
    _init: function()
    {
        PanelMenu.Button.prototype._init.call(this, 0.0);
        this._iconActor = new St.Icon({ icon_name: 'document-open-recent',
                                        icon_type: St.IconType.SYMBOLIC,
                                        style_class: 'system-status-icon' });
        this.actor.add_actor(this._iconActor);
        this.actor.add_style_class_name('panel-status-button');

        this.RecentManager = new DocInfo.DocManager();
        this._display();

        this.RecentManager.connect('changed', Lang.bind(this, this._redisplay));

        Main.panel.addToStatusArea('recent-items', this);
    },

   _display: function()
   {
        let id = 0;
        let idshow = 0;
        let blacklistString = BLACKLIST.replace(/\s/g, "");
        let blacklistList = blacklistString.split(",");

        while (idshow < ITEMS && id < this.RecentManager._infosByTimestamp.length)
        {
            if (blacklistList.indexOf((this.RecentManager._infosByTimestamp[id].mimeType.split("/"))[0]) == -1)
            {
                let icon = this.RecentManager._infosByTimestamp[id].createIcon(22);
                let menuItem = new MyPopupMenuItem(icon, this.RecentManager._infosByTimestamp[id].name, {});
                this.menu.addMenuItem(menuItem);
                menuItem.connect('activate', Lang.bind(this, this._launchFile, this.RecentManager._infosByTimestamp[id].uri));
                idshow++;
            }
            id++;
        }

        if (id < this.RecentManager._infosByTimestamp.length && MORE > 0)
        {
            this.moreItem = new PopupMenu.PopupSubMenuMenuItem(_("More..."));
            this.menu.addMenuItem(this.moreItem);
            while (idshow < ITEMS+MORE && id < this.RecentManager._infosByTimestamp.length)
            {
                if (blacklistList.indexOf((this.RecentManager._infosByTimestamp[id].mimeType.split("/"))[0]) == -1)
                {
                    let icon = this.RecentManager._infosByTimestamp[id].createIcon(22);
                    let menuItem = new MyPopupMenuItem(icon, this.RecentManager._infosByTimestamp[id].name, {});
                    this.moreItem.menu.addMenuItem(menuItem);
                    menuItem.connect('activate', Lang.bind(this, this._launchFile, this.RecentManager._infosByTimestamp[id].uri));
                    idshow++;
                }
                id++;
            }
        }

        if (this.RecentManager._infosByTimestamp.length > 0)
        {
            this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
            let icon = new St.Icon({ icon_name: 'edit-clear', icon_type: St.IconType.SYMBOLIC, icon_size: 22 });
            let menuItem = new MyPopupMenuItem(icon, 'Clear list', {});
            this.menu.addMenuItem(menuItem);
            menuItem.connect('activate', Lang.bind(this, this._clearAll));
        }
    },
    _redisplay: function()
    {
        this.menu.removeAll();
        this._display();
    },
    _launchFile: function(a, b, c)
    {
        Gio.app_info_launch_default_for_uri(c, global.create_app_launch_context());
    },
    _clearAll: function()
    {
        let GtkRecent = new Gtk.RecentManager();
        GtkRecent.purge_items();
    },
    destroy: function()
    {
        this.RecentManager.disconnectAll();
        this.actor._delegate = null;
        this.menu.destroy();
        this.actor.destroy();
        this.emit('destroy');
    }
};

function init()
{
}

let Rec;

function enable()
{
  Rec = new RecentItems();
}
function disable()
{
  Rec.destroy();
}

